
////////////////////////////////////////////////////////////////
//                                                            //
//  Header file for class InDetProjHelper                     //
//                                                            //
//  Description: Helper class for calculating projections     //
//               of data to inner detector surfaces.          //
//                                                            //
//  Author: Thomas H. Kittelmann (Thomas.Kittelmann@cern.ch)  //
//  Initial version: February 2008                            //
//                                                            //
////////////////////////////////////////////////////////////////

#ifndef INDETPROJHELPER_H
#define INDETPROJHELPER_H

#include "VP1Base/VP1HelperClassBase.h"
#include "VP1Utils/InDetProjFlags.h"
#include "CLHEP/Geometry/Point3D.h"
#include <vector>
#include <set>

class InDetProjHelper : public VP1HelperClassBase {
public:

  //Use one of the following to create an appropriate helper:
  static InDetProjHelper * createPixelHelper(IVP1System* sys = 0);
  static InDetProjHelper * createSCTHelper(IVP1System* sys = 0);
  static InDetProjHelper * createTRTHelper(IVP1System* sys = 0);
  //(non-zero system pointers to let error messages appear in gui)

  virtual ~InDetProjHelper();

  InDetProjFlags::InDetProjPartsFlags setParts(InDetProjFlags::InDetProjPartsFlags);//returns old value
  InDetProjFlags::InDetProjPartsFlags parts() const;

  //Clip path to volumes of the enabled detector parts, as given by the parts flag.
  void clipPath( const std::vector<HepPoint3D>& path, std::set<std::vector<HepPoint3D> >& resulting_subpaths ) const;
  void clipPath( const std::vector<HepPoint3D>& path,
		 std::set<std::vector<HepPoint3D> >& resulting_subpaths_barrelA,
		 std::set<std::vector<HepPoint3D> >& resulting_subpaths_barrelC,
		 std::set<std::vector<HepPoint3D> >& resulting_subpaths_endcapA,
		 std::set<std::vector<HepPoint3D> >& resulting_subpaths_endcapC ) const;

  //Clip path as above, and then project the resulting subpaths to surfaces as given by the current parts flag.
  void projectPath( const std::vector<HepPoint3D>& path, std::set<std::vector<HepPoint3D> >& resulting_projections ) const;
  void projectPath( const std::vector<HepPoint3D>& path,
		    std::set<std::vector<HepPoint3D> >& resulting_projections_barrelA,
		    std::set<std::vector<HepPoint3D> >& resulting_projections_barrelC,
		    std::set<std::vector<HepPoint3D> >& resulting_projections_endcapA,
		    std::set<std::vector<HepPoint3D> >& resulting_projections_endcapC ) const;

  //To figure out which parts a given track crosses (this does NOT depend on the current InDetProjPartsFlags setting).
  enum PartsFlag { NoParts = 0x00, BarrelA = 0x01, BarrelC = 0x02, EndCapA = 0x04, EndCapC = 0x08 };
  Q_DECLARE_FLAGS(PartsFlags, PartsFlag)
  PartsFlags touchedParts( const std::vector<HepPoint3D>& path ) const;

  //Fixme: Projection of points? Projection of driftcircle (get normal and point? Or just get transform?)

  static void transformECPointToZPlane_specialZtoR(HepPoint3D& p,
						   const double& planeZ,
						   const double& planeRBegin,
						   const double& endcapZBegin,
						   const double& squeezeFactor );

private:

  InDetProjHelper( const double& surfacethickness,
		   const double& data_disttosurface_epsilon,
		   const double& barrel_inner_radius,
		   const double& barrel_outer_radius,
		   const double& barrel_posneg_z,
		   const double& endcap_surface_z,
		   const double& endcap_surface_length,
		   const double& endcap_inner_radius,
		   const double& endcap_outer_radius,
		   const double& endcap_zasr_innerradius,
		   const double& endcap_zasr_endcapz_begin,
		   const double& endcap_zasr_squeezefact,
		   IVP1System* sys );



  class Imp;
  Imp * d;

};

Q_DECLARE_OPERATORS_FOR_FLAGS(InDetProjHelper::PartsFlags)

#endif
