
////////////////////////////////////////////////////////////////
//                                                            //
//  Implementation of class VP1EtaCone                        //
//                                                            //
//  Author: Thomas H. Kittelmann (Thomas.Kittelmann@cern.ch)  //
//  Initial version: April 2008                               //
//                                                            //
////////////////////////////////////////////////////////////////

#include "VP1GuideLineSystems/VP1EtaCone.h"
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include "CLHEP/Units/SystemOfUnits.h"

//____________________________________________________________________
class VP1EtaCone::Imp {
public:
  Imp(VP1EtaCone *,
      SoMaterial * mat,
      SoSeparator * attachsep);
  VP1EtaCone * theclass;
  SoMaterial * material;
  SoSeparator * attachSep;

  bool shown;
  double etaval;
  double extent;

  SoSeparator * sep;
  SoCone * cone1;
  SoTranslation * trans1;
  SoCone * innercone1;
  SoTranslation * innertrans1;
  SoCone * cone2;
  SoTranslation * trans2;
  SoCone * innercone2;
  SoTranslation * innertrans2;

  void updateFields();
  void ensureInit3DObjects();
};

//____________________________________________________________________
VP1EtaCone::VP1EtaCone(SoMaterial * mat,SoSeparator * attachsep,
		       IVP1System * sys,QObject * parent)
  : QObject(parent), VP1HelperClassBase(sys,"VP1EtaCone"), d(new Imp(this,mat,attachsep))
{
}

//____________________________________________________________________
VP1EtaCone::~VP1EtaCone()
{
  setShown(false);
  if (d->sep)
    d->sep->unref();
  d->material->unref();
  d->attachSep->unref();
  delete d;
}

//____________________________________________________________________
VP1EtaCone::Imp::Imp(VP1EtaCone *tc,SoMaterial * mat,SoSeparator * as)
  : theclass(tc), material(mat), attachSep(as), shown(false),
    etaval(1), extent(1), sep(0),
    cone1(0), trans1(0), innercone1(0), innertrans1(0),
    cone2(0), trans2(0), innercone2(0), innertrans2(0)
{
  material->ref();
  attachSep->ref();
}

//____________________________________________________________________
void VP1EtaCone::Imp::ensureInit3DObjects()
{
  if (sep)
    return;
  theclass->messageVerbose("Building 3D objects");
  sep = new SoSeparator;
  sep->ref();
  sep->addChild(material);

  SoSeparator * sepfirst = new SoSeparator;
  SoSeparator * sepsecond = new SoSeparator;

  for ( int i = 0;i<2;++i){
    SoRotationXYZ * xf = new SoRotationXYZ();
    xf->axis=SoRotationXYZ::X;
    xf->angle = i==0 ? 90.0*deg : -90*deg;
    SoTranslation  * xl = new SoTranslation();
    SoCone * cone = new SoCone();
    SoTranslation  * innerxl = new SoTranslation();
    SoCone * innercone = new SoCone();

    if (i==0) {
      cone1 = cone;
      trans1 = xl;
      innercone1 = innercone;
      innertrans1 = innerxl;
    } else {
      cone2 = cone;
      trans2 = xl;
      innercone2 = innercone;
      innertrans2 = innerxl;
    }

    cone->removePart(SoCone::BOTTOM);
    innercone->removePart(SoCone::BOTTOM);

    SoSeparator * s(i==0?sepfirst:sepsecond);
    s->addChild(xf);
    s->addChild(xl);
    s->addChild(cone);
    s->addChild(innerxl);
    s->addChild(innercone);
    sep->addChild(s);
  }
}

//____________________________________________________________________
void VP1EtaCone::Imp::updateFields()
{
  ensureInit3DObjects();
  theclass->messageVerbose("Updating fields");

  double theta = 2*atan(exp(-fabs(etaval)));
  double tanfact = tan(theta);
  bool etaconeextentisz = (extent<0);
  double absextent = fabs(extent);
  double coneHeight,bottomRadius;
  if (etaconeextentisz) {
    coneHeight = absextent;
    bottomRadius = coneHeight*tanfact;
  } else {
    bottomRadius = absextent;
    coneHeight=bottomRadius/tanfact;
  }

  theclass->messageVerbose("etaval = "+str(etaval));
  theclass->messageVerbose("extent = "+str(extent));
  theclass->messageVerbose("absextent = "+str(absextent));
  theclass->messageVerbose("tanfact = "+str(tanfact));
  theclass->messageVerbose("bottomRadius = "+str(bottomRadius));
  theclass->messageVerbose("coneHeight = "+str(coneHeight));

  cone1->bottomRadius = bottomRadius;
  cone1->height = coneHeight;
  cone2->bottomRadius = bottomRadius;
  cone2->height = coneHeight;
  trans1->translation.setValue(0, -coneHeight/2, 0);
  trans2->translation.setValue(0, -coneHeight/2, 0);

  innercone1->bottomRadius = bottomRadius;
  innercone1->height = -coneHeight;
  innercone2->bottomRadius = bottomRadius;
  innercone2->height = -coneHeight;
  //To avoid render flicker from the overlapping cones, we move the inner cone slightly away from origo:
  const double epsilon = 0.50*mm;
  innertrans1->translation.setValue(0, coneHeight/2+1.001*coneHeight/2+epsilon, 0);
  innertrans2->translation.setValue(0, coneHeight/2+1.001*coneHeight/2+epsilon, 0);

}

//____________________________________________________________________
void VP1EtaCone::setShown(bool b)
{
  messageVerbose("Signal received: setShown("+str(b)+")");
  if (d->shown==b)
    return;
  d->shown=b;
  if (d->shown) {
    d->updateFields();
    if (d->attachSep->findChild(d->sep)<0)
      d->attachSep->addChild(d->sep);
  } else {
    if (d->sep&&d->attachSep->findChild(d->sep)>=0)
      d->attachSep->removeChild(d->sep);
  }
}

//____________________________________________________________________
void VP1EtaCone::setExtent(const double&p)
{
  messageVerbose("Signal received: setExtent("+str(p)+")");
  if (d->extent==p)
    return;
  d->extent=p;
  if (d->shown)
    d->updateFields();
}

//____________________________________________________________________
void VP1EtaCone::setEtaValue(const double&p)
{
  messageVerbose("Signal received: setEtaValue("+str(p)+")");
  if (d->etaval==p)
    return;
  d->etaval=p;
  if (d->shown)
    d->updateFields();
}
